@@ -5,6 +5,7 @@ import io
 import logging
 import os
 import tempfile
+import uuid
 from contextlib import redirect_stdout
 
 import cbor2
@@ -20,6 +21,7 @@ import pycardano
 
 import uplc
 import uplc.ast
+from uplc.cost_model import PlutusVersion
 
 from . import (
     compiler,
@@ -33,6 +35,7 @@ from . import (
 from .util import CompilerError, data_from_json, OPSHIN_LOG_HANDLER
 from .prelude import ScriptContext
 from .compiler_config import *
+from uplc import cost_model
 
 
 class Command(enum.Enum):
@@ -142,7 +145,7 @@ def plutus_data_from_json(annotation: typing.Type, x: dict):
                     }
         if issubclass(annotation, pycardano.PlutusData):
             return annotation.from_dict(x)
-    except (KeyError, ValueError):
+    except (KeyError, ValueError, pycardano.DeserializeException):
         raise ValueError(
             f"Annotation {annotation} does not match provided plutus datum {json.dumps(x)}"
         )
@@ -188,7 +191,7 @@ def plutus_data_from_cbor(annotation: typing.Type, x: bytes):
                 )
         if issubclass(annotation, pycardano.PlutusData):
             return annotation.from_cbor(x)
-    except (KeyError, ValueError):
+    except (KeyError, ValueError, pycardano.DeserializeException):
         raise ValueError(
             f"Annotation {annotation} does not match provided plutus datum {x.hex()}"
         )
@@ -196,52 +199,42 @@ def plutus_data_from_cbor(annotation: typing.Type, x: bytes):
 
 def check_params(
     command: Command,
-    purpose: Purpose,
-    validator_args,
-    return_type,
-    validator_params,
-    force_three_params=False,
+    validator_args: typing.List[typing.Tuple[str, typing.Type]],
+    return_type: typing.Type,
+    validator_params: typing.List,
+    parameters: int,
 ):
-    num_onchain_params = (
-        3
-        if purpose == Purpose.spending or force_three_params or purpose == Purpose.any
-        else 2
-    )
+    num_onchain_params = 1
     onchain_params = validator_args[-num_onchain_params:]
     param_types = validator_args[:-num_onchain_params]
-    if purpose == Purpose.any:
-        # The any purpose does not do any checks. Use only if you know what you are doing
-        return onchain_params, param_types
-    # expect the validator to return None
-    assert (
-        return_type is None or return_type == prelude.Anything
-    ), f"Expected contract to return None, but returns {return_type}"
-
-    required_onchain_parameters = 3 if purpose == Purpose.spending else 2
-    if force_three_params:
-        datum_type = onchain_params[0][1]
-        assert (
-            (
-                typing.get_origin(datum_type) == typing.Union
-                and prelude.Nothing in typing.get_args(datum_type)
-            )
-            or datum_type == prelude.Anything
-            or datum_type == prelude.Nothing
-        ), f"Expected contract to accept Nothing or Anything as datum since it forces three parameters, but got {datum_type}"
+    if return_type is not None:
+        print(
+            f"Warning: validator returns {return_type}, but it is recommended to return None. In PlutusV3, validators that do not return None always fail. This is most likely not what you want."
+        )
 
+    required_onchain_parameters = 1
     assert (
         len(onchain_params) == required_onchain_parameters
     ), f"""\
-{purpose.value.capitalize()} validator must expect {required_onchain_parameters} parameters at evaluation (on-chain), but was specified to have {len(onchain_params)}.
-Make sure the validator expects parameters {'datum, ' if purpose == Purpose.spending else ''}redeemer and script context."""
+The validator must expect {required_onchain_parameters} parameters at evaluation (on-chain), but was specified to have {len(onchain_params)}.
+Make sure the validator expects exactly the script context (ScriptContext)."""
 
     if command in (Command.eval, Command.eval_uplc):
         assert len(validator_params) == len(param_types) + len(
             onchain_params
-        ), f"{purpose.value.capitalize()} validator expects {len(param_types) + len(onchain_params)} parameters for evaluation, but only got {len(validator_params)}."
+        ), f"The validator expects {len(param_types) + len(onchain_params)} parameters for evaluation, but only got {len(validator_params)}."
+    else:
+        assert (
+            len(param_types) - len(validator_params) == parameters
+        ), f"""\
+The validator is specified to expect {len(param_types)-len(validator_params)} parameters for parameterization ({','.join(x[0] for x in param_types)}, of which {len(validator_params)} are bound by additional arguments), but the command line arguments indicate {parameters} parameters (default is 0).
+Note that PlutusV3 validatators expect only the ScriptContext as on-chain parameter, so non-parameterized contracts should have only 1 parameter ({onchain_params[0][0]}).
+Make sure the number of parameters passed matches the number of parameters expected by the validator.
+Either remove extra parameters or specify them by passing `--parameters {len(param_types)-len(validator_params)}`.
+"""
     assert (
         onchain_params[-1][1] == ScriptContext
-    ), f"Last parameter of the validator has to be ScriptContext, but is {onchain_params[-1][1].__name__} here."
+    ), f"Last parameter of the validator ({onchain_params[-1][0]}) has to be of type ScriptContext, but is {onchain_params[-1][1].__name__}."
     return onchain_params, param_types
 
 
@@ -257,20 +250,17 @@ def perform_command(args):
     # configure logging
     if args.verbose:
         OPSHIN_LOG_HANDLER.setLevel(logging.DEBUG)
+    lib = args.lib
+    number_parameters = args.parameters
 
     # execute the command
     command = Command(args.command)
-    purpose = Purpose(args.purpose)
-    if purpose == Purpose.lib:
-        assert (
-            not compiler_config.remove_dead_code
-        ), "Libraries must have dead code removal disabled (-fno-remove-dead-code)"
     input_file = args.input_file if args.input_file != "-" else sys.stdin
     # read and import the contract
     with open(input_file, "r") as f:
         source_code = f.read()
     with tempfile.TemporaryDirectory(prefix="build") as tmpdir:
-        tmp_input_file = pathlib.Path(tmpdir).joinpath("__tmp_opshin.py")
+        tmp_input_file = pathlib.Path(tmpdir).joinpath(f"__tmp_opshin{uuid.uuid4()}.py")
         with tmp_input_file.open("w") as fp:
             fp.write(source_code)
         sys.path.append(str(pathlib.Path(tmp_input_file).parent.absolute()))
@@ -283,67 +273,51 @@ def perform_command(args):
             ) from e
         sys.path.pop()
     # load the passed parameters if not a lib
-    if purpose == Purpose.lib:
-        assert not args.args, "Can not pass arguments to a library"
-        parsed_params = []
-        uplc_params = []
-    else:
-        try:
-            argspec = inspect.signature(sc.validator)
-        except AttributeError:
-            can_command_with_lib = command in (
-                Command.compile,
-                Command.build,
-                Command.compile_pluto,
-                Command.parse,
-                Command.lint,
-            )
-            command_with_lib = f" or {command.value} using `opshin {command.value} lib {str(input_file)}`."
-            raise AssertionError(
-                f"Contract has no function called 'validator'. Make sure the compiled contract contains one function called 'validator'"
-                + (command_with_lib if can_command_with_lib else ".")
-            )
-        annotations = [
-            (x.name, x.annotation or prelude.Anything)
-            for x in argspec.parameters.values()
-        ]
-        return_annotation = (
-            argspec.return_annotation
-            if argspec.return_annotation not in (None, argspec.empty)
-            else prelude.Anything
+    try:
+        argspec = inspect.signature(sc.validator if lib is None else getattr(sc, lib))
+    except AttributeError:
+        raise AssertionError(
+            f"Contract has no function called '{'validator' if lib is None else lib}'. Make sure the compiled contract contains one function called 'validator'."
         )
-        parsed_params = []
-        uplc_params = []
-        for i, (c, a) in enumerate(zip(annotations, args.args)):
-            try:
-                uplc_param = parse_uplc_param(a)
-            except ValueError as e:
-                raise ValueError(
-                    f"Could not parse parameter {i} ('{a}') as UPLC data. Please provide the parameter either as JSON or CBOR (in hexadecimal notation). Detailed error: {e}"
-                ) from None
-            uplc_params.append(uplc_param)
-            try:
-                param = parse_plutus_param(c[1], a)
-            except ValueError as e:
-                raise ValueError(
-                    f"Could not parse parameter {i} ('{a}') as type {c[1]}. Please provide the parameter either as JSON or CBOR (in hexadecimal notation). Detailed error: {e}"
-                ) from None
-            parsed_params.append(param)
+    annotations = [
+        (x.name, x.annotation or prelude.Anything) for x in argspec.parameters.values()
+    ]
+    return_annotation = (
+        argspec.return_annotation
+        if argspec.return_annotation is not argspec.empty
+        else prelude.Anything
+    )
+    parsed_params = []
+    uplc_params = []
+    for i, (c, a) in enumerate(zip(annotations, args.args)):
+        try:
+            uplc_param = parse_uplc_param(a)
+        except ValueError as e:
+            raise ValueError(
+                f"Could not parse parameter {i} ('{a}') as UPLC data. Please provide the parameter either as JSON or CBOR (in hexadecimal notation). Detailed error: {e}"
+            ) from None
+        uplc_params.append(uplc_param)
+        try:
+            param = parse_plutus_param(c[1], a)
+        except ValueError as e:
+            raise ValueError(
+                f"Could not parse parameter {i} ('{a}') as type {c[1]}. Please provide the parameter either as JSON or CBOR (in hexadecimal notation). Detailed error: {e}"
+            ) from None
+        parsed_params.append(param)
+    if lib is None:
         onchain_params, param_types = check_params(
             command,
-            purpose,
             annotations,
             return_annotation,
             parsed_params,
-            compiler_config.force_three_params,
+            number_parameters,
         )
         assert (
-            onchain_params or purpose == Purpose.lib
-        ), "The validator function must have at least one on-chain parameter for non-library contracts."
+            onchain_params
+        ), "The validator function must have at least one on-chain parameter. You can also add `_:None`."
 
     py_ret = Command.eval
     if command == Command.eval:
-        assert purpose != Purpose.lib, "Can not evaluate a library"
         print("Python execution started")
         with redirect_stdout(open(os.devnull, "w")):
             try:
@@ -362,7 +336,7 @@ def perform_command(args):
         code = compiler.compile(
             source_ast,
             filename=input_file,
-            validator_function_name="validator" if purpose != Purpose.lib else None,
+            validator_function_name="validator" if lib is None else lib,
             # do not remove dead code when compiling a library - none of the code will be used
             config=compiler_config,
         )
@@ -396,7 +370,7 @@ Note that opshin errors may be overly restrictive as they aim to prevent code wi
             # we remove chaining so that users to not see the internal trace back,
         )
         err.orig_err = c.orig_err
-        raise err from None
+        raise err
 
     if command == Command.compile_pluto:
         print(code.dumps())
@@ -415,6 +389,10 @@ Note that opshin errors may be overly restrictive as they aim to prevent code wi
         return
 
     if command == Command.build:
+        if lib is not None:
+            raise ValueError(
+                "Cannot build a library. Please remove the --lib flag when building a contract."
+            )
         if args.output_directory == "":
             if args.input_file == "-":
                 print(
@@ -425,20 +403,14 @@ Note that opshin errors may be overly restrictive as they aim to prevent code wi
         else:
             target_dir = pathlib.Path(args.output_directory)
         built_code = builder._build(code)
-        if purpose == Purpose.lib:
-            script_arts = PlutusContract(
-                built_code,
-                title=pathlib.Path(input_file).stem,
-            )
-        else:
-            script_arts = PlutusContract(
-                built_code,
-                datum_type=onchain_params[0] if len(onchain_params) == 3 else None,
-                redeemer_type=onchain_params[1 if len(onchain_params) == 3 else 0],
-                parameter_types=param_types,
-                purpose=(purpose,),
-                title=pathlib.Path(input_file).stem,
-            )
+        script_arts = PlutusContract(
+            built_code,
+            datum_type=onchain_params[0] if len(onchain_params) == 3 else None,
+            redeemer_type=onchain_params[1 if len(onchain_params) == 3 else 0],
+            parameter_types=param_types,
+            purpose=(Purpose.any,),
+            title=pathlib.Path(input_file).stem,
+        )
         script_arts.dump(target_dir)
 
         print(f"Wrote script artifacts to {target_dir}/")
@@ -446,7 +418,11 @@ Note that opshin errors may be overly restrictive as they aim to prevent code wi
     if command == Command.eval_uplc:
         print("UPLC execution started")
         assert isinstance(code, uplc.ast.Program)
-        raw_ret = uplc.eval(code)
+        raw_ret = uplc.eval(
+            code,
+            cek_machine_cost_model=cost_model.default_cek_machine_cost_model_plutus_v3(),
+            builtin_cost_model=cost_model.default_builtin_cost_model_plutus_v3(),
+        )
         print("------LOGS--------")
         if raw_ret.logs:
             for log in raw_ret.logs:
@@ -476,22 +452,18 @@ def parse_args():
         type=str,
         choices=Command.__members__.keys(),
         help="The command to execute on the input file.",
-        default="eval",
-        nargs="?",
-    )
-    a.add_argument(
-        "purpose",
-        type=str,
-        choices=Purpose.__members__.keys(),
-        help="The intended script purpose. Determines the number of on-chain parameters "
-        "(spending = 3, minting, rewarding, certifying = 2, any = no checks). "
-        "This allows the compiler to check whether the correct amount of parameters was passed during compilation.",
-        default="any",
-        nargs="?",
     )
     a.add_argument(
         "input_file", type=str, help="The input program to parse. Set to - for stdin."
     )
+    a.add_argument(
+        "--lib",
+        const="validator",
+        default=None,
+        nargs="?",
+        type=str,
+        help="Indicates that the input file should compile to a generic function, reusable by other contracts (not a smart contract itself). Discards corresponding typechecks. An optional name of the function to export can be given, by default it is validator. Use -fwrap_input and -fwrap_output to control whether the function expects or returns PlutusData (otherwise BuiltIn).",
+    )
     a.add_argument(
         "-o",
         "--output-directory",
@@ -503,13 +475,19 @@ def parse_args():
         "args",
         nargs="*",
         default=[],
-        help="Input parameters for the validator (parameterizes the contract for compile/build). Either json or CBOR notation.",
+        help="Input parameters for the validator (parameterizes the contract for compile/build, passes the values for eval/eval_uplc). Either json or CBOR notation.",
     )
     a.add_argument(
         "--output-format-json",
         action="store_true",
         help="Changes the output of the Linter to a json format.",
     )
+    a.add_argument(
+        "--parameters",
+        type=int,
+        default=0,
+        help="Number of parameters that the contract supports. If not specified, 0 is assumed (i.e., the contract only expects the ScriptContext).",
+    )
     a.add_argument(
         "--version",
         action="version",
@@ -523,7 +501,7 @@ def parse_args():
     )
     a.add_argument(
         "--recursion-limit",
-        default=sys.getrecursionlimit(),
+        default=max(sys.getrecursionlimit(), 4000),
         help="Modify the recursion limit (necessary for larger UPLC programs)",
         type=int,
     )
