@@ -8,10 +8,11 @@ import pluthon
 class CompilationConfig(pluthon.CompilationConfig):
     constant_folding: Optional[bool] = None
     allow_isinstance_anything: Optional[bool] = None
-    force_three_params: Optional[bool] = None
     remove_dead_code: Optional[bool] = None
     fast_access_skip: Optional[int] = None
     expand_union_types: Optional[bool] = None
+    wrap_output: Optional[bool] = None
+    unwrap_input: Optional[bool] = None
 
 
 # The default configuration for the compiler
@@ -41,15 +42,16 @@ OPT_O2_CONFIG = (
     )
 )
 OPT_O3_CONFIG = (
-    CompilationConfig().update(pluthon.OPT_O3_CONFIG).update(OPT_O2_CONFIG).update()
+    CompilationConfig().update(OPT_O2_CONFIG).update(pluthon.OPT_O3_CONFIG).update()
 )
 OPT_CONFIGS = [OPT_O0_CONFIG, OPT_O1_CONFIG, OPT_O2_CONFIG, OPT_O3_CONFIG]
 
 DEFAULT_CONFIG = CompilationConfig(
     allow_isinstance_anything=False,
-    force_three_params=False,
     expand_union_types=False,
-).update(OPT_O1_CONFIG)
+    wrap_output=False,
+    unwrap_input=True,
+).update(OPT_O2_CONFIG)
 
 ARGPARSE_ARGS = pluthon.ARGPARSE_ARGS.copy()
 ARGPARSE_ARGS.update(
@@ -61,10 +63,6 @@ ARGPARSE_ARGS.update(
         "allow_isinstance_anything": {
             "help": "Enables the use of isinstance(x, D) in the contract where x is of type Anything. This is not recommended as it only checks the constructor id and not the actual type of the data.",
         },
-        "force_three_params": {
-            "__alts__": ["--ftp"],
-            "help": "Enforces that the contract is always called with three virtual parameters on-chain. Enable if the script should support spending and other purposes.",
-        },
         "remove_dead_code": {
             "help": "Removes dead code and variables from the contract. Should be enabled for non-debugging purposes.",
         },
@@ -76,6 +74,14 @@ ARGPARSE_ARGS.update(
             "__alts__": ["--eut"],
             "help": "Expand functions with Union type arguments into monomorphic variants (e.g. foo(Union[int, bytes]) -> foo_i(int), foo_b(bytes)). This should allow the compiler to optimise away redundant type checks when argument types are known at compile time. This is an O3-level optimisation and may increase script size significantly.",
         },
+        "wrap_output": {
+            "__alts__": ["--wo"],
+            "help": "Wraps the output of the validator in PlutusData. This is useful for exporting library functions that return non-None values as validators.",
+        },
+        "unwrap_input": {
+            "__alts__": ["--wi"],
+            "help": "Unwraps the input of the validator from PlutusData. Disabling this is useful for exporting library functions that take builtin data as input.",
+        },
     }
 )
 for k in ARGPARSE_ARGS:
