@@ -1,10 +1,6 @@
-import re
-from copy import copy
-from typing import Optional
-from enum import Enum
-
 from ..util import CompilingNodeTransformer
 from ..typed_ast import *
+import pluthon as plt
 
 """
 Replaces empty dicts with UPLC constants of empty data pairs
