@@ -1,5 +1,7 @@
 """An implementation of some math operations in opshin"""
 
+from opshin.std.builtins import *
+
 
 def gcd(a: int, b: int) -> int:
     """Returns the greatest common divisor of a and b"""
@@ -19,26 +21,16 @@ def unsigned_int_from_bytes_big(b: bytes) -> int:
 
     For example, the bytestring b'\\x01\\x02\\x03' will be converted to the integer 66051.
     """
-    acc = 0
-    for i in range(len(b)):
-        acc = acc * 256 + b[i]
-    return acc
+    return byte_string_to_integer(True, b)
 
 
 def bytes_big_from_unsigned_int(b: int) -> bytes:
     """
-    Converts an integer into the corresponding bytestring, big/network byteorder, unsigned
+    Converts an integer into the corresponding bytestring of minimal width, big/network byteorder, unsigned
 
     For example, the integer 66051 will be converted to the bytestring b'\\x01\\x02\\x03'.
     """
-    assert b >= 0
-    if b == 0:
-        return b"\x00"
-    acc = b""
-    while b > 0:
-        acc = bytes([b % 256]) + acc
-        b //= 256
-    return acc
+    return integer_to_byte_string(True, 0, b)
 
 
 def ceil(a: int, b: int):
@@ -49,3 +41,8 @@ def ceil(a: int, b: int):
 def floor(a: int, b: int):
     """Returns a divided by b rounded towards negative infinity"""
     return a // b
+
+
+def and_bytestring(a: bytes, b: bytes) -> bytes:
+    """Returns the bitwise AND of two bytestrings, padding the shorter one with trailing ones"""
+    return and_byte_string(True, a, b)
