@@ -157,21 +157,28 @@ def constant_type(c):
         return StringInstanceType
     if isinstance(c, list):
         assert len(c) > 0, "Lists must be non-empty"
-        first_typ = constant_type(c[0])
-        assert all(
-            constant_type(ce) == first_typ for ce in c[1:]
-        ), "Constant lists must contain elements of a single type only"
+        types = [constant_type(x) for x in c]
+        first_typ = find_max_type([InstanceType(t) for t in types])
+        if first_typ is None:
+            raise ValueError(
+                f"All elements in a list must have a compatible type, found typs {tuple(t.python_type() for t in types)}"
+            )
         return InstanceType(ListType(first_typ))
     if isinstance(c, dict):
         assert len(c) > 0, "Dicts must be non-empty"
-        first_key_typ = constant_type(next(iter(c.keys())))
-        first_value_typ = constant_type(next(iter(c.values())))
-        assert all(
-            constant_type(ce) == first_key_typ for ce in c.keys()
-        ), "Constant dicts must contain keys of a single type only"
-        assert all(
-            constant_type(ce) == first_value_typ for ce in c.values()
-        ), "Constant dicts must contain values of a single type only"
+
+        key_types = [constant_type(k) for k in c.keys()]
+        value_types = [constant_type(v) for v in c.values()]
+        first_key_typ = find_max_type([InstanceType(t) for t in key_types])
+        first_value_typ = find_max_type([InstanceType(t) for t in value_types])
+        if first_key_typ is None:
+            raise ValueError(
+                f"All keys in a dict must have a compatible type, found typs {tuple(t.python_type() for t in key_types)}"
+            )
+        if first_value_typ is None:
+            raise ValueError(
+                f"All values in a dict must have a compatible type, found typs {tuple(t.python_type() for t in value_types)}"
+            )
         return InstanceType(DictType(first_key_typ, first_value_typ))
     if isinstance(c, PlutusData):
         return InstanceType(RecordType(record=record_from_plutusdata(c)))
@@ -791,8 +798,8 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
         for t in node.targets:
             assert isinstance(
                 t, Name
-            ), "Can only assign to variable names (e.g., x = 5). OpShin does not allow assigning to tuple deconstructors (e.g., a, b = (1, 2)) or to dicts, lists, or members (e.g., x[0] = 1; x.foo = 1)"
-            # Check compatability to previous types -> variable can be bound in a function before and needs to maintain type
+            ), "Can only assign to variable names (e.g., x = 5 or a, b = 10, 20). OpShin does not allow assigning to dicts, lists, or members (e.g., x[0] = 1; x.foo = 1)"
+            # Check compatibility to previous types -> variable can be bound in a function before and needs to maintain type
             self.set_variable_type(t.id, typed_ass.value.typ)
         typed_ass.targets = [self.visit(t) for t in node.targets]
         # for deconstructed tuples, check that the size matches
@@ -837,7 +844,7 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
         assert isinstance(
             node.target, Name
         ), "Can only assign to variable names, no type deconstruction"
-        # Check compatability to previous types -> variable can be bound in a function before and needs to maintain type
+        # Check compatibility to previous types -> variable can be bound in a function before and needs to maintain type
         self.set_variable_type(node.target.id, InstanceType(typed_ass.annotation))
         typed_ass.target = self.visit(node.target)
         assert (
@@ -1056,7 +1063,14 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
         tb = copy(node)
         tb.left = self.visit(node.left)
         tb.right = self.visit(node.right)
-        binop_fun_typ: FunctionType = tb.left.typ.binop_type(tb.op, tb.right.typ)
+        try:
+            binop_fun_typ: FunctionType = tb.left.typ.binop_type(tb.op, tb.right.typ)
+        except NotImplementedError as e:
+            try:
+                # attempt reverse binop
+                binop_fun_typ = tb.right.typ.rbinop_type(tb.op, tb.left.typ)
+            except NotImplementedError:
+                raise e
         tb.typ = binop_fun_typ.rettyp
 
         return tb
@@ -1455,7 +1469,7 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
 
     def visit_ListComp(self, node: ListComp) -> TypedListComp:
         typed_listcomp = copy(node)
-        # inside the comprehension is a seperate scope
+        # inside the comprehension is a separate scope
         self.enter_scope()
         # first evaluate generators for assigned variables
         typed_listcomp.generators = [self.visit(s) for s in node.generators]
@@ -1485,7 +1499,7 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
 
     def visit_DictComp(self, node: DictComp) -> TypedDictComp:
         typed_dictcomp = copy(node)
-        # inside the comprehension is a seperate scope
+        # inside the comprehension is a separate scope
         self.enter_scope()
         # first evaluate generators for assigned variables
         typed_dictcomp.generators = [self.visit(s) for s in node.generators]
